"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const taskWrapper_1 = require("./taskWrapper");
const cron_parser_1 = require("cron-parser");
const ms = require("ms");
const taskRunner_1 = require("./taskRunner");
class Task {
    constructor(taskName, taskRoot, packageManager) {
        this.taskRoot = taskRoot;
        this.taskName = taskName;
        this.initialized = false;
        this.history = [];
        this.packageManager = packageManager;
    }
    restartTimer() {
        if (this.initialized) {
            if (this.schedule.includes(' ')) {
                const interval = cron_parser_1.parseExpression(this.schedule);
                const cronRun = () => {
                    const now = Date.now();
                    const next = interval.next().getTime();
                    this.timer = setTimeout(() => {
                        this.run();
                        cronRun();
                    }, next - now);
                };
            }
            else {
                this.timer = setInterval(() => {
                    this.run();
                }, ms(this.schedule));
            }
        }
    }
    stopTimer() {
        if (this.timer) {
            clearInterval(this.timer);
            clearTimeout(this.timer);
        }
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.initialized) {
                return;
            }
            else {
                yield this.reload();
            }
        });
    }
    reload() {
        return __awaiter(this, void 0, void 0, function* () {
            this.stopTimer();
            const { task, version } = yield taskRunner_1.default.runTask({ code: taskWrapper_1.configParser(this.taskName), virtualFilePath: path_1.resolve(this.taskRoot, `${this.taskName}.js`) });
            this.version = version;
            this.status = 0 /* pending */;
            this.schedule = task.schedule;
            this.timeout = task.timeout;
            this.singleton = !!task.singleton;
            this.initialized = true;
            this.restartTimer();
        });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.init();
            yield this.updateSelf();
            const startTime = Date.now();
            if (this.singleton && this.status === 1 /* running */) {
                this.history.push({ type: 2 /* singletonCanceled */, startTime, finishedTime: Date.now() });
            }
            else {
                try {
                    this.status = 1 /* running */;
                    const data = yield taskRunner_1.default.runTask({ code: taskWrapper_1.runner(this.taskName), virtualFilePath: path_1.resolve(this.taskRoot, `${this.taskName}.js`), data: this.data });
                    this.data = data;
                    this.status = 0 /* pending */;
                    this.history.push({ type: 0 /* succeed */, startTime, finishedTime: Date.now() });
                }
                catch (e) {
                    this.status = 0 /* pending */;
                    this.history.push({ type: 1 /* error */, startTime, finishedTime: Date.now(), error: e });
                }
            }
            if (this.history.length > 20) {
                this.history = this.history.slice(this.history.length - 20);
            }
        });
    }
    updateSelf() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.packageManager.installPackage(this.taskName);
        });
    }
}
exports.Task = Task;
