"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const PackageManager_1 = require("./PackageManager");
const Task_1 = require("./Task");
class TaskServer {
    constructor(options) {
        this.tasks = {};
        const { packageStorePath, npmRegistry, packages } = options;
        this.packageStorePath = packageStorePath;
        this.packageManager = new PackageManager_1.PackageManager({ registry: npmRegistry });
        this.packages = packages;
    }
    _installInitialTasks() {
        return __awaiter(this, void 0, void 0, function* () {
            let packages = this.packages;
            if (this.packages.then) {
                this.packages = packages = yield this.packages;
            }
            return yield this.packageManager.installPackages(packages);
        });
    }
    _registerTask(taskName) {
        if (this.tasks[taskName]) {
            return;
        }
        this.tasks[taskName] = new Task_1.Task(taskName, this.packageStorePath, this.packageManager);
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._installInitialTasks();
            this.packages.forEach(name => this._registerTask(name));
        });
    }
}
exports.TaskServer = TaskServer;
