import { ChildProcess } from 'child_process';
import { Task as TaskClass } from '../src/Task';

type timestamp = number;

export interface Options {
  // newThread: boolean;
  packageStorePath: string;
  packages: Array<string> | Promise<Array<string>>
  npmRegistry: string;
}

declare namespace Task {

  export const enum Status {
    pending,
    running,
  }

  export interface Base {
    schedule: string; // e.g. '1h', '* * 3 * * *'
    timeout?: string | number ; // e.g '5m', 1000 * 60 * 5
    singleton?: boolean;
  }

  export interface Registered extends Base {
    status: Status;
    timer: NodeJS.Timer;
    history: Array<History.Record>;
    version: string;
    data: any;
  }

  export interface Registry {
    [packageName: string]: TaskClass;
  }

}

declare namespace History {

  export const enum Type {
    succeed,
    error,
    singletonCanceled,
  }

  export interface Record {
    type: Type,
    startTime: number;
    finishedTime: number;
    error?: Error;
  }

}

// declare namespace VM {
//
//   export interface Result {
//     type: 'finished' | 'canceled' | 'error',
//     data?: any,
//     error?: Error,
//   }
//
//   export interface Runner {
//     (): Promise<Result>;
//   }
// }

